// -------------------------------------- //
// Peng Guo, Indiana University, Jan.2015 //
// -------------------------------------- //


#include <cassert>
#include <iostream>
#include <fstream>
#include <string>
#include <complex>
#include <cstdlib>
#include "2bamp.h"

#include <gsl/gsl_math.h>
#include <gsl/gsl_errno.h>
#include <gsl/gsl_integration.h>
#include <gsl/gsl_spline.h>
#include <gsl/gsl_interp.h>
#include <gsl/gsl_sf_coupling.h>
#include <gsl/gsl_sf_legendre.h>









// amplitudes in terms of (x,y) //


complex<double>	TwobodyAmp::amp_xy(double x, double y, int m1, int m2, int m3,int Imax, int Lmax, double meta, double mpip, double mpim, double mpi0, struct param_space *pa0_IL) 
{
    double Qeta = meta - mpip - mpim - mpi0;
    
    double metasq = meta*meta;
    double mpipsq = mpip*mpip;
    double mpimsq = mpim*mpim;
    double mpi0sq = mpi0*mpi0;
    
    
    
    double s = pow(meta-mpi0,2) - 2.0*meta*Qeta/3.0*(y+1.0);
    double t = meta*Qeta/sqrt(3.0)*x + (metasq+mpipsq+mpimsq+mpi0sq-s)/2.0;
    double u = metasq+mpipsq+mpimsq+mpi0sq-s-t;
    
    double phi = s*t*u - u*(mpi0sq*mpipsq + metasq*mpimsq)
    - s*(mpipsq*mpimsq + metasq*mpi0sq)
    - t*(mpimsq*mpi0sq + metasq*mpipsq)
    + 2.0*mpi0sq*mpipsq*mpimsq + 2.0*metasq*(mpipsq*mpimsq + mpipsq*mpi0sq + mpi0sq*mpimsq);

    
    
    complex<double>	 r (0.0,0.0);
    
    if (phi>0.0)
    {
        r = allamp(s,t,m1,m2,m3,Imax,Lmax,meta,mpip,mpim,mpi0,pa0_IL);
    }
    
    return r;
}





complex<double>	TwobodyAmp::allamp(double s, double t, int m1, int m2, int m3,int Imax, int Lmax, double meta, double mpip, double mpim, double mpi0, struct param_space *pa0_IL)
{
    complex<double>	 r (0.0,0.0);
    
    for (int i=1; i<=paramfitnum; i++)
    {
        int Ip = a0fit_I[i];
        int Lp = a0fit_L[i];
        r = r + (*pa0_IL).a0_I_L_struct[Ip][Lp]*totamp(s,t,m1,m2,m3,Imax,Lmax,Ip,Lp,meta,mpip,mpim,mpi0);
    }
    
    return r;
}









complex<double>	TwobodyAmp::totamp(double s, double t, int m1, int m2, int m3,int Imax, int Lmax, int Ip, int Lp, double meta, double mpip, double mpim, double mpi0) const
{
    complex<double>	 r (0.0,0.0);
    
    if (int(1.0+pow(-1.0,Ip+Lp))!=0)
    {
        double metasq = meta*meta;
        double mpipsq = mpip*mpip;
        double mpimsq = mpim*mpim;
        double mpi0sq = mpi0*mpi0;
        
        double u = metasq + mpipsq + mpimsq + mpi0sq - s - t;
        
        double phi = s*t*u - u*(mpi0sq*mpipsq + metasq*mpimsq)
        - s*(mpipsq*mpimsq + metasq*mpi0sq)
        - t*(mpimsq*mpi0sq + metasq*mpipsq)
        + 2.0*mpi0sq*mpipsq*mpimsq + 2.0*metasq*(mpipsq*mpimsq + mpipsq*mpi0sq + mpi0sq*mpimsq);
        
        
        
        
        if (phi>0.0)
        {
            double cos_s = - ( 2.0*s*( metasq + mpipsq - t ) - (s + mpipsq - mpimsq)*(metasq + s - mpi0sq) )/
            ( sqrt( (s-pow(mpip-mpim,2))*(s-pow(mpip+mpim,2)))*sqrt((s-pow(meta-mpi0,2))*(s-pow(meta+mpi0,2)) ) );
            
            double cos_t = ( 2.0*t*( metasq + mpimsq - u ) - (t + mpimsq - mpi0sq)*(metasq + t -mpipsq) )/
            ( sqrt( (t-pow(mpim-mpi0,2))*(t-pow(mpim+mpi0,2)))*sqrt((t-pow(meta-mpip,2))*(t-pow(meta+mpip,2)) ) );
            
            double cos_u =  ( 2.0*u*( metasq + mpi0sq - s ) - (u + mpi0sq - mpipsq)*(metasq +u-mpimsq) )/
            ( sqrt( (u-pow(mpi0-mpip,2))*(u-pow(mpi0+mpip,2)))*sqrt((u-pow(meta-mpim,2))*(u-pow(meta+mpim,2)) ) );
            
            
            for (int Is=0; Is<=Imax; Is++)
            {
                for (int L=0; L<=Lmax; L++)
                {
                    if (int(1.0+pow(-1.0,Is+L))!=0)
                    {
                        r = r + pow(-1.0,Is)*(2.0*L+1.0)*gsl_sf_legendre_Pl(L,cos_s)*amp_PW(s,m1,m2,m3,Is,L,Ip,Lp, meta,mpip,mpim,mpi0)
                        + (2.0*L+1.0)*gsl_sf_legendre_Pl(L,cos_t)*amp_PW(t,m2,m3,m1,Is,L,Ip,Lp, meta,mpim,mpi0,mpip)
                        + (2.0*L+1.0)*gsl_sf_legendre_Pl(L,cos_u)*amp_PW(u,m3,m1,m2,Is,L,Ip,Lp, meta,mpi0,mpip,mpim);
                    }
                }
            }
            
        }
    }
    
    
    return r;
}








complex<double>	TwobodyAmp::amp_PW(double s,int m1, int m2, int m3, int Is, int L, int Ip, int Lp, double meta, double mass1, double mass2, double mass3) const
{
    complex<double>	 r (0.0,0.0);
    
    if (int(1.0+pow(-1.0,Is+L))!=0)
    {
        if ((Is==Ip)&&(L==Lp))
        {
            r = IS_coef(Is,m1,m2,m3)*pow(s*sqrt(Ksq(s,meta,mass1,mass2,mass3)),L)*thatIL(s,Is,L); // new barrier //
        }
        else
        {
            r = 0.0;
        }
    }
    
    return r;
}













// computing parameter space for given process //

void	TwobodyAmp::parameterspace(int m1, int m2, int m3, int Imax, int Lmax)
{
    paramfitnum = 0;
    
    cout << "****************************************************" << endl;
    cout << " Determing fitting parameters space a[I][L] for process: " << endl;
    cout << " eta -> pi[" << m1 << "]+pi["<< m2 << "]+pi["<< m3 << "]." << endl;
    
    cout << " non-zero a[I][L] are: " << endl;
    cout << endl;
    
    for (int Is=0; Is<=Imax; Is++)
    {
        for (int L=0; L<=Lmax; L++)
        {
            if ((int(1.0+pow(-1.0,Is+L))!=0)&&((IS_coef(Is,m1,m2,m3)!=0.0)||(IS_coef(Is,m2,m3,m1)!=0.0)||(IS_coef(Is,m3,m1,m2)!=0.0)))
            {
                cout << " a["<<Is<<"]["<< L << "] " << endl;
                paramfitnum = paramfitnum + 1 ;
                a0fit_I[paramfitnum] = Is;
                a0fit_L[paramfitnum] = L;
            }
        }
    }
    cout << endl;
    
    cout << "************************************************" << endl;
}

// ========================================== //







// initialization of fitting parameters //


void	TwobodyAmp::parameters_ini(string inputfile, int m1, int m2, int m3, int Imax, int Lmax,  struct param_space *pa0_IL)
{
    ifstream inpars(inputfile.c_str());
    if (inpars.fail())
    {
        cout  << "Error Opening File: " << inputfile << endl;
        exit(1);
    }
    
    
	for (int Is=0; Is<=2; Is++)
	{
        for (int L=0; L<=2; L++)
        {
            string paranames;
            inpars >> paranames >> (*pa0_IL).a0_I_L_struct[Is][L];
        }
	}
	inpars.close();
    
}
// ==================================== //


// -------------------------------------- //
// Peng Guo, Indiana University, Jan.2015 //
// -------------------------------------- //


