// -------------------------------------- //
// Peng Guo, Indiana University, Jan.2015 //
// -------------------------------------- //


#ifndef _THREEBODYAMP_
#define _THREEBODYAMP_


#include <utility>
#include <string>
#include <complex>
#include <vector>

#include "pipiamp_params.h"
#include "pipiamps.h"
#include "cvert_and_basicfuncs.h"

using std::complex;
using namespace std;






struct param3b_space
{
    double a0_I_L_struct[10][10];
};






class ThreebodyAmp :  public PipiAmp
{

public:

    
    
    ThreebodyAmp() :  PipiAmp()
    {
        ndimright = 601;
        s3bsub = 0.0255;
        
        metafit = 0.549;
        
        sgfitup = 0.165;
        sgfitlow = 0.14;
        fitdim = 5;
    }

  ~ThreebodyAmp(){}


    complex<double>	amp_xy_3b(double x, double y, int m1, int m2, int m3,int Imax, int Lmax,  double meta, double mpip, double mpim, double mpi0, struct param3b_space *pa0_IL);
    
    complex<double>	allamp3b(double s, double t, int m1, int m2, int m3,int Imax, int Lmax,  double meta, double mpip, double mpim, double mpi0, struct param3b_space *pa0_IL);
    
    complex<double>	totamp3b(double s, double t, int m1, int m2, int m3,int Imax, int Lmax, int Ip, int Lp,  double meta, double mpip, double mpim, double mpi0) const;
    
    complex<double>	amp_PW_3b(double s,int m1, int m2, int m3, int Is, int L, int Ip, int Lp,  double meta,double mass1, double mass2, double mass3) const;
    
    
    
    
    
    complex<double>	Delta0t(double t, double meta) const;
    complex<double>	Delta1t(double t, double meta) const;
    complex<double>	Delta2t(double t, double meta) const;
    complex<double>	Delta00(double s,double t, double meta) const;
    complex<double>	Delta01(double s,double t, double meta) const;
    complex<double>	Delta10(double s,double t, double meta) const;
    complex<double>	Delta11(double s,double t, double meta) const;
    complex<double>	DeltaL1L2(int L1, int L2, double s,double t, double meta) const;
    
    
    complex<double>	NewDeltaL1L2(int L1, int L2, double s,double t, double meta) const;
    
    complex<double>	AdlerZeroDeltaL1L2(int L1, int L2, double s,double t, double meta, int I1) const;
    
    // ---------------- //
    complex<double>	Delta3st(double s, double t, double meta) const;
    complex<double>	Delta3tplus(double t, double meta) const;
    complex<double>	Delta3tminus(double t, double meta) const;
    complex<double>	Delta02tplus(double t, double meta) const;
    complex<double>	Delta02tminus(double t, double meta) const;
    double Phi_st(double s, double t, double meta) const;
    complex<double>	Delta4t(double t, double meta) const;
    complex<double>	Delta5t(double t, double meta) const;
    complex<double>	Delta02(double s,double t, double meta) const;
    complex<double>	Delta12(double s,double t, double meta) const;
    complex<double>	Delta00A(double s,double t, double meta) const;
    complex<double>	Delta00B(double s,double t, double meta) const;
    complex<double>	Delta00C(double s,double t, double meta) const;
    complex<double>	Delta20A(double s,double t, double meta) const;
    complex<double>	Delta20B(double s,double t, double meta) const;
    complex<double>	Delta20C(double s,double t, double meta) const;
    complex<double>	Delta20D(double s,double t, double meta) const;
    complex<double>	Delta20E(double s,double t, double meta) const;
    complex<double>	Delta20F(double s,double t, double meta) const;
    complex<double>	Delta20(double s,double t, double meta) const;
    
    complex<double>	NewDelta20A(double s,double t, double meta) const;
    complex<double>	NewDelta20(double s,double t, double meta) const;
    
    complex<double>	Delta21A(double t, double meta) const;
    complex<double>	Delta21B(double t, double meta) const;
    complex<double>	Delta21(double s,double t, double meta) const;
    
    complex<double>	NewDelta21A(double t, double meta) const;
    complex<double>	NewDelta21B(double t, double meta) const;
    complex<double>	NewDelta21(double s,double t, double meta) const;

    complex<double>	Delta22A(double s,double t, double meta) const;
    complex<double>	Delta22B(double t, double meta) const;
    complex<double>	Delta22(double s,double t, double meta) const;
    
    complex<double>	NewDelta22A(double s,double t, double meta) const;
    complex<double>	NewDelta22B(double t, double meta) const;
    complex<double>	NewDelta22(double s,double t, double meta) const;
    // ---------------- //
    
    
    complex<double>  mat(int nmat, int i, int mmat, int j, int *pstore_I, int *pstore_L, double meta) const;
    complex<double>  mat_I_L_ij(int Is, int Ls, int It, int Lt,int i, int j, double meta) const;
    void invertmatrix(int nmatsize, int *pstore_I, int *pstore_L, int n,complex<double> *aaa , double meta) const;
    
    
    void	GhatLinv(int Imax, int Lmax, int It, int Lt, double meta) const;
    
    complex<double>	T3bhatIL(double s,int I, int L, int Ip, int Lp, double meta) const;
   
    
    
    
    
    
    
    void fitG(int Is, int Ls, int It, int Lt, int nreim);
    void fitGit(int m1, int m2, int m3, int Imax, int Lmax);
    complex<double> GILfit(int Is, int Ls, int It, int Lt, double s) const;
    
  
    
    
    
    complex<double>	Ubelow(double s, double meta) const;
    complex<double>	Splus(double s, double meta) const;
    complex<double>	Sminus(double s, double meta) const;
    
    

    
    void	parameterspace3b(int m1, int m2, int m3, int Imax, int Lmax);
    void	parameters_ini_3b(string inputfile, int m1, int m2, int m3, int Imax, int Lmax,  struct param3b_space *pa0_IL);
    
    
    int paramfitnum3b ;
    int a0fit3b_I[10];
    int a0fit3b_L[10];
    
    
    
    // discretization of s,t //
    int ndimright ;
    double xradright[5100];
    double weradright[5100];
    
    double s3bsub;
    
    
    
    double metafit;
    
    double sgfitup;
    double sgfitlow;


    int fitdim;
    
    
private:
	

};

#endif



// -------------------------------------- //
// Peng Guo, Indiana University, Jan.2015 //
// -------------------------------------- //
