// -------------------------------------- //
// Peng Guo, Indiana University, Jan.2015 //
// -------------------------------------- //

#ifndef _BASICFUNCS_
#define _BASICFUNCS_


#include <utility>
#include <string>
#include <complex>
#include <vector>


#include <gsl/gsl_math.h>
#include <gsl/gsl_errno.h>
#include <gsl/gsl_integration.h>
#include <gsl/gsl_spline.h>
#include <gsl/gsl_interp.h>
#include <gsl/gsl_sf_coupling.h>
#include <gsl/gsl_sf_legendre.h>



using std::complex;
using namespace std;






class BasicFuncs
{
    
public:
    
    BasicFuncs() { }
    
    ~BasicFuncs(){}
    
    
    double IS_coef(int t, int m1, int m2, int m3) const;
    double CG_coef(int j1, int j2, int j3, int m1, int m2, int m3) const;
    
    double	Ksq(double s, double meta,double mass1, double mass2, double mass3) const;
    
    complex<double>	cpsqrt(complex<double> tc) const;
    void gaulegf(double x1, double x2, double x[], double w[], int n) const;
    
    
    
    
    
    
    
    // matrix inversion subroutines
	
	void   zgedi ( complex <double> a[], int lda, int n, int ipvt[], complex <double> det[2], int job ) const;
	double zabs1 ( complex <double> z ) const;
	double r8_abs ( double x ) const;
	void zscal ( int n, complex <double> ca, complex <double> cx[], int incx ) const;
	void zaxpy ( int n, complex <double> ca, complex <double> cx[],int incx, complex <double> cy[], int incy ) const;
    void zswap ( int n, complex <double> cx[], int incx, complex <double> cy[], int incy ) const;
	double zgeco ( complex <double> a[], int lda, int n, int ipvt[] ) const;
	complex <double> zsign1 ( complex <double> z1, complex <double> z2 ) const;
	void zdscal ( int n, double sa, complex <double> cx[], int incx ) const;
	complex <double> zdotc ( int n, complex <double> cx[], int incx, complex <double> cy[], int incy ) const;
	double dzasum ( int n, complex <double> x[], int incx ) const;
	int zgefa ( complex <double> a[], int lda, int n, int ipvt[] ) const;
	int izamax ( int n, complex <double> x[], int incx ) const;
	double r8_max ( double x, double y ) const;
	
	// end of matrix inversion subroutines
    
    
    
    double enlg3(double *x,double *y,int n,double t) const;
    double enatk(double *x,double *y,int n,double t, double eps) const;
    
    
    
private:
	
    
};

#endif


// -------------------------------------- //
// Peng Guo, Indiana University, Jan.2015 //
// -------------------------------------- //
