
// -------------------------------------- //
// Peng Guo, Indiana University, Jan.2015 //
// -------------------------------------- //

#include <cassert>
#include <iostream>
#include <string>
#include <complex>
#include <cstdlib>

#include "pipiamps.h"





// pi pi parametrization //


// Is=0, S-wave //

double	PipiAmp::delta00_low(double s) const
{
	double EM = 0.85;
    double cotd;
    if (sqrt(s)<=EM)
    {
        double z0 = mpi;
        double w  = (sqrt(s) - sqrt(4.0*mK*mK-s))/(sqrt(s) + sqrt(4.0*mK*mK-s));
        
        double b0 = getarraypipi(0);
        double b1 = getarraypipi(1);
        double b2 = getarraypipi(2);
        double b3 = getarraypipi(3);
        
        cotd = 1.0/sqrt(1.0-4.0*mpi*mpi/s)*mpi*mpi/(s-z0*z0/2.0)*(z0*z0/mpi/sqrt(s)+b0+b1*w+b2*w*w+b3*w*w*w);
    }
    
    double r = 0.0 ;
    if (sqrt(s)<=EM)
    {
        r = atan(1.0/cotd)/PI*180.0;
    }
    
    if (r<0.0)
    {
        r = r + 180.0;
    }
    
	return r;
}




double	PipiAmp::delta00_upp(double s) const
{
	double EM = 0.85;
    double eps = 0.0000000001;
    
    
    double r = 0.0 ;
    if ((sqrt(s)>EM)&&(sqrt(s)<=1.42))
    {
        if (sqrt(s)<2.0*mK)
        {
            double kK = sqrt(4.0*mK*mK-s)/2.0;
            double kM = sqrt(4.0*mK*mK-EM*EM)/2.0;
            double d0 = 227.1;
            double dM = delta00_low(EM*EM);
            double ddM = (delta00_low(EM*EM)-delta00_low(EM*EM-eps))/eps;
            double c  = -660;
            
            r = d0*pow(1.0-kK/kM,2) + dM*kK/kM*(2.0-kK/kM) + kK*(kM-kK)*(8.0*ddM+c*(kM-kK)/mK/mK/mK);
        }
        else
        {
            double kK = sqrt(s-4.0*mK*mK)/2.0;
            double d0 = 227.1;
            double B  = 94.0;
            double C  = 40.4;
            double D  = -86.9;
            
            r = d0 + B*pow(kK/mK,2) + C*pow(kK/mK,4) ;
            
            double meta = 0.54751;
            if (sqrt(s)>2.0*meta)
            {
                double keta = sqrt(s-4.0*meta*meta)/2.0;
                r = r + D*pow(keta/meta,2);
            }
        }
    }
    
	return r;
}






double	PipiAmp::delta00(double s) const
{
    double EM = 0.85;
    
    double r = 0.0 ;
    if (sqrt(s)<=EM)
    {
        r = delta00_low(s);
    }
    else if (sqrt(s)<=1.42)
    {
        r = delta00_upp(s);
    }
    
    
	return r;
}




double	PipiAmp::eta00(double s) const
{
    
    double r = 1.0 ;
    if ((sqrt(s)>2.0*mK)&&(sqrt(s)<=1.42))
    {
        double kK = sqrt(s-4.0*mK*mK)/2.0;
        double e1 = 4.7;
        double e2 = -15.0;
        double e3 = 4.7;
        double e4 = 0.38;
        
        r = - kK/sqrt(s)*pow(e1+e2*kK/sqrt(s)+e3*kK*kK/s,2) ;
        
        double meta = 0.54751;
        if (sqrt(s)>2.0*meta)
        {
            double keta = sqrt(s-4.0*meta*meta)/2.0;
            r = r - e4*keta/sqrt(s);
        }
        
        r = exp(r);
    }
    
	return r;
}








complex<double>	PipiAmp::t00(double s) const
{
	complex<double> r (0.0,0.0);
    
    if (s<=4.0*mpi*mpi)
    {
        double z0 = mpi;
        double b0 =  7.26;
        double b1 = -25.3;
        double b2 = -33.1;
        double b3 = -26.6;
        
        complex<double> w (0.0,0.0);
        complex<double> cotd (0.0,0.0);
        
        if (s>=0.0)
        {
            w = (sqrt(s) - sqrt(4.0*mK*mK-s))/(sqrt(s) + sqrt(4.0*mK*mK-s));
            cotd = 1.0/aIm/sqrt(4.0*mpi*mpi/s-1.0)*mpi*mpi/(s-z0*z0/2.0)*(z0*z0/mpi/sqrt(s)+b0+b1*w+b2*w*w+b3*w*w*w);
            r = 1.0/aIm/sqrt(4.0*mpi*mpi/s-1.0)/(cotd - aIm);
        }
        else
        {
            w = (aIm*sqrt(-s) - sqrt(4.0*mK*mK-s))/(aIm*sqrt(-s) + sqrt(4.0*mK*mK-s));
            cotd = 1.0/sqrt(1.0-4.0*mpi*mpi/s)*mpi*mpi/(s-z0*z0/2.0)*(z0*z0/mpi/aIm/sqrt(-s)+b0+b1*w+b2*w*w+b3*w*w*w);
            r = 1.0/sqrt(1.0-4.0*mpi*mpi/s)/(cotd - aIm);
        }
    }
    else if (sqrt(s)<=1.42)
    {
        r = 1.0/sqrt(1.0-4.0*mpi*mpi/s)*(eta00(s)*exp(2.0*aIm*delta00(s)/180.0*PI)-1.0)/2.0/aIm;
    }
    
    
	return r;
}

// =================================== //














// Is=1, P-wave //

double	PipiAmp::delta11_low(double s) const
{
    double cotd;
    if (sqrt(s)<=2.0*mK)
    {
        double kpi = sqrt(s-4.0*mpi*mpi)/2.0;
        double w  = (sqrt(s) - sqrt(1.05*1.05-s))/(sqrt(s) + sqrt(1.05*1.05-s));
        
        double b0 = getarraypipi(4);
        double b1 = getarraypipi(5);
        
        cotd = 1.0/sqrt(1.0-4.0*mpi*mpi/s)*(mrho*mrho-s)/kpi/kpi*(2.0*mpi*mpi*mpi/mrho/mrho/sqrt(s)+b0+b1*w);
    }
    
    double r = 0.0 ;
    if (sqrt(s)<=2.0*mK)
    {
        r = atan(1.0/cotd)/PI*180.0;
    }
    
    if (r<0.0)
    {
        r = r + 180.0;
    }
    
	return r;
}




double	PipiAmp::delta11_upp(double s) const
{
    double r = 0.0 ;
    if ((sqrt(s)>2.0*mK)&&(sqrt(s)<=1.42))
    {
        double lamb0 = delta11_low(4.0*mK*mK);
        double lamb1 = 1.57/PI*180.0;
        double lamb2 = -1.96/PI*180.0;
        
        r = lamb0 + lamb1*(sqrt(s)/2.0/mK-1.0) + lamb2*pow(sqrt(s)/2.0/mK-1.0,2);
    }
    
	return r;
}





double	PipiAmp::delta11(double s) const
{
    double r = 0.0 ;
    if (sqrt(s)<=2.0*mK)
    {
        r = delta11_low(s);
    }
    else if (sqrt(s)<=1.42)
    {
        r = delta11_upp(s);
    }
    
	return r;
}





double	PipiAmp::eta11(double s) const
{
    double r = 1.0 ;
    if ((sqrt(s)>2.0*mK)&&(sqrt(s)<=1.42))
    {
        double e1 = 0.1;
        double e2 = 0.11;
        
        r = 1.0 - e1*sqrt(1.0-4.0*mK*mK/s) - e2*(1.0-4.0*mK*mK/s) ;
    }
    
	return r;
}






complex<double>	PipiAmp::t11(double s) const
{
	complex<double> r (0.0,0.0);
    
    if (s<=4.0*mpi*mpi)
    {
        double b0 =  1.055;
        double b1 =  0.15;
        
        complex<double> kpi  (0.0,0.0);
        complex<double> w    (0.0,0.0);
        complex<double> cotd (0.0,0.0);
        
        if (s>=0.0)
        {
            kpi = aIm*sqrt(4.0*mpi*mpi-s)/2.0;
            w  = (sqrt(s) - sqrt(1.05*1.05-s))/(sqrt(s) + sqrt(1.05*1.05-s));
            cotd = 1.0/aIm/sqrt(4.0*mpi*mpi/s-1.0)*(mrho*mrho-s)/kpi/kpi*(2.0*mpi*mpi*mpi/mrho/mrho/sqrt(s)+b0+b1*w);
            r = 1.0/aIm/sqrt(4.0*mpi*mpi/s-1.0)/(cotd - aIm);
        }
        else
        {
            kpi = aIm*sqrt(4.0*mpi*mpi-s)/2.0;
            w = (aIm*sqrt(-s) - sqrt(1.05*1.05-s))/(aIm*sqrt(-s) + sqrt(1.05*1.05-s));
            cotd = 1.0/sqrt(1.0-4.0*mpi*mpi/s)*(mrho*mrho-s)/kpi/kpi*(2.0*mpi*mpi*mpi/mrho/mrho/aIm/sqrt(-s)+b0+b1*w);
            r = 1.0/sqrt(1.0-4.0*mpi*mpi/s)/(cotd - aIm);
        }
    }
    else if (sqrt(s)<=1.42)
    {
        r = 1.0/sqrt(1.0-4.0*mpi*mpi/s)*(eta11(s)*exp(2.0*aIm*delta11(s)/180.0*PI)-1.0)/2.0/aIm;
    }
    
    
	return r;
}



// =================================== //










// Is=0, D-wave //

double	PipiAmp::delta20_low(double s) const
{
    double cotd;
    if (sqrt(s)<=2.0*mK)
    {
        double kpi = sqrt(s-4.0*mpi*mpi)/2.0;
        double w  = (sqrt(s) - sqrt(1.05*1.05-s))/(sqrt(s) + sqrt(1.05*1.05-s));
        
        double b0 = getarraypipi(8);
        double b1 = getarraypipi(9);
        
        
        cotd = 1.0/sqrt(1.0-4.0*mpi*mpi/s)*(mf2*mf2-s)/pow(kpi,4)*mpi*mpi*(b0+b1*w);
    }
    
    double r = 0.0 ;
    if (sqrt(s)<=2.0*mK)
    {
        r = atan(1.0/cotd)/PI*180.0;
    }
    
    if (r<0.0)
    {
        r = r + 180.0;
    }
    
	return r;
}




double	PipiAmp::delta20_upp(double s) const
{
    double cotd;
    if ((sqrt(s)>2.0*mK)&&(sqrt(s)<=1.42))
    {
        double kpi = sqrt(s-4.0*mpi*mpi)/2.0;
        double wh  = (sqrt(s) - sqrt(1.45*1.45-s))/(sqrt(s) + sqrt(1.45*1.45-s));
        double b1h =  43.7;
        
        double b0 =  12.47;
        double b1 =  10.12;
        double wK  = (2.0*mK - sqrt(1.05*1.05-4.0*mK*mK))/(2.0*mK + sqrt(1.05*1.05-4.0*mK*mK));
        double whK  = (2.0*mK - sqrt(1.45*1.45-4.0*mK*mK))/(2.0*mK + sqrt(1.45*1.45-4.0*mK*mK));
        
        double b0h = b0 + b1*wK - b1h*whK;
        
        cotd = 1.0/sqrt(1.0-4.0*mpi*mpi/s)*(mf2*mf2-s)/pow(kpi,4)*mpi*mpi*(b0h+b1h*wh);
    }
    
    double r = 0.0 ;
    if ((sqrt(s)>2.0*mK)&&(sqrt(s)<=1.42))
    {
        r = atan(1.0/cotd)/PI*180.0;
    }
    
    
    if (r<0.0)
    {
        r = r + 180.0;
    }
    
	return r;
}







double	PipiAmp::delta20(double s) const
{
    double r = 0.0 ;
    if (sqrt(s)<=2.0*mK)
    {
        r = delta20_low(s);
    }
    else if (sqrt(s)<=1.42)
    {
        r = delta20_upp(s);
    }
    
    
	return r;
}




double	PipiAmp::eta20(double s) const
{
    
    double eta = 1.0 ;
    if ((sqrt(s)>2.0*mK)&&(sqrt(s)<=1.42))
    {
        double kK  = sqrt(s-4.0*mK*mK)/2.0;
        double kM2 = sqrt(mf2*mf2-4.0*mK*mK)/2.0;
        double e  = 0.284;
        double r  = 2.54;
        
        
        eta = 1.0 -e*pow( (1.0-4.0*mK*mK/s)/(1.0-4.0*mK*mK/mf2/mf2),5.0/2.0)*( 1.0+r*(1.0-kK/kM2) );
    }
    
    
    
	return eta;
}








complex<double>	PipiAmp::t20(double s) const
{
	complex<double> r (0.0,0.0);
    
    if (s<=4.0*mpi*mpi)
    {
        double b0 =  12.47;
        double b1 =  10.12;
        
        
        complex<double> kpi  (0.0,0.0);
        complex<double> w    (0.0,0.0);
        complex<double> cotd (0.0,0.0);
        
        if (s>=0.0)
        {
            kpi = aIm*sqrt(4.0*mpi*mpi-s)/2.0;
            w  = (sqrt(s) - sqrt(1.05*1.05-s))/(sqrt(s) + sqrt(1.05*1.05-s));
            cotd = 1.0/aIm/sqrt(4.0*mpi*mpi/s-1.0)*(mf2*mf2-s)/pow(kpi,4)*mpi*mpi*(b0+b1*w);
            r = 1.0/aIm/sqrt(4.0*mpi*mpi/s-1.0)/(cotd - aIm);
        }
        else
        {
            kpi = aIm*sqrt(4.0*mpi*mpi-s)/2.0;
            w = (aIm*sqrt(-s) - sqrt(1.05*1.05-s))/(aIm*sqrt(-s) + sqrt(1.05*1.05-s));
            cotd = 1.0/sqrt(1.0-4.0*mpi*mpi/s)*(mf2*mf2-s)/pow(kpi,4)*mpi*mpi*(b0+b1*w);
            r = 1.0/sqrt(1.0-4.0*mpi*mpi/s)/(cotd - aIm);
        }
    }
    else if (sqrt(s)<=1.42)
    {
        r = 1.0/sqrt(1.0-4.0*mpi*mpi/s)*(eta20(s)*exp(2.0*aIm*delta20(s)/180.0*PI)-1.0)/2.0/aIm;
    }
    
    
	return r;
}


// =============================== //





















// I=2, S-wave //


double	PipiAmp::delta02_low(double s) const
{
	double EM = 0.85;
    double cotd;
    if (sqrt(s)<=EM)
    {
        double z0 = mpi;
        double w  = (sqrt(s) - sqrt(1.05*1.05-s))/(sqrt(s) + sqrt(1.05*1.05-s));
        
        double b0 = getarraypipi(6);
        double b1 = getarraypipi(7);
        
        cotd = 1.0/sqrt(1.0-4.0*mpi*mpi/s)*mpi*mpi/(s-2.0*z0*z0)*(b0+b1*w);
    }
    
    double r = 0.0 ;
    if (sqrt(s)<=EM)
    {
        r = atan(1.0/cotd)/PI*180.0;
    }
    
	return r;
}




double	PipiAmp::delta02_upp(double s) const
{
	double EM = 0.85;
    double cotd;
    
    if ((sqrt(s)>EM)&&(sqrt(s)<=1.42))
    {
        double z0 = mpi;
        double wh  = (sqrt(s) - sqrt(1.42*1.42-s))/(sqrt(s) + sqrt(1.42*1.42-s));
        double whEM  = (EM - sqrt(1.42*1.42-EM*EM))/(EM + sqrt(1.42*1.42-EM*EM));
        double b0 = -80.4;
        double b1 = -73.6;
        double bh0 = b0+b1*(EM - sqrt(1.05*1.05-EM*EM))/(EM + sqrt(1.05*1.05-EM*EM));
        double bh1 = b1*(1.05*1.05)/(1.42*1.42)*sqrt((1.42*1.42-EM*EM)/(1.05*1.05-EM*EM))*pow((EM+sqrt(1.42*1.42-EM*EM))/(EM+sqrt(1.05*1.05-EM*EM)),2);
        double bh2 = 112.0;
        
        
        cotd = 1.0/sqrt(1.0-4.0*mpi*mpi/s)*mpi*mpi/(s-2.0*z0*z0)*(bh0+bh1*(wh-whEM)+bh2*pow(wh-whEM,2));
    }
    
    double r = 0.0 ;
    if ((sqrt(s)>EM)&&(sqrt(s)<=1.42))
    {
        r = atan(1.0/cotd)/PI*180.0;
    }
    
	return r;
}







double	PipiAmp::delta02(double s) const
{
    double EM = 0.85;
    
    double r = 0.0 ;
    if (sqrt(s)<=EM)
    {
        r = delta02_low(s);
    }
    else if (sqrt(s)<=1.42)
    {
        r = delta02_upp(s);
    }
    
    
	return r;
}






double	PipiAmp::eta02(double s) const
{
    
    double r = 1.0 ;
    if ((sqrt(s)>1.05)&&(sqrt(s)<=1.42))
    {
        double e = 0.28;
        
        r = 1.0-e*pow(1.0-1.05*1.05/s,3.0/2.0) ;
    }
    
    
	return r;
}








complex<double>	PipiAmp::t02(double s) const
{
	complex<double> r (0.0,0.0);
    
    if (s<=4.0*mpi*mpi)
    {
        double z0 = mpi;
        double b0 = -80.4;
        double b1 = -73.6;
        
        complex<double> w (0.0,0.0);
        complex<double> cotd (0.0,0.0);
        
        if (s>=0.0)
        {
            w = (sqrt(s) - sqrt(1.05*1.05-s))/(sqrt(s) + sqrt(1.05*1.05-s));
            cotd = 1.0/aIm/sqrt(4.0*mpi*mpi/s-1.0)*mpi*mpi/(s-2.0*z0*z0)*(b0+b1*w);
            r = 1.0/aIm/sqrt(4.0*mpi*mpi/s-1.0)/(cotd - aIm);
        }
        else
        {
            w = (aIm*sqrt(-s) - sqrt(1.05*1.05-s))/(aIm*sqrt(-s) + sqrt(1.05*1.05-s));
            cotd = 1.0/sqrt(1.0-4.0*mpi*mpi/s)*mpi*mpi/(s-2.0*z0*z0)*(b0+b1*w);
            r = 1.0/sqrt(1.0-4.0*mpi*mpi/s)/(cotd - aIm);
        }
    }
    else if (sqrt(s)<=1.42)
    {
        r = 1.0/sqrt(1.0-4.0*mpi*mpi/s)*(eta02(s)*exp(2.0*aIm*delta02(s)/180.0*PI)-1.0)/2.0/aIm;
    }
    
    
	return r;
}



// ============================== //








// I=2, D-wave //


double	PipiAmp::delta22(double s) const
{
    double cotd;
    if (sqrt(s)<=1.42)
    {
        double kpi = sqrt(s-4.0*mpi*mpi)/2.0;
        double w  = (sqrt(s) - sqrt(1.45*1.45-s))/(sqrt(s) + sqrt(1.45*1.45-s));
        
        double b0 = getarraypipi(10);
        double b1 = getarraypipi(11);
        double b2 = getarraypipi(12);
        double Delta = getarraypipi(13);
        
        cotd = 1.0/sqrt(1.0-4.0*mpi*mpi/s)/pow(kpi,4)*pow(mpi,4)*s/(4.0*(mpi*mpi+Delta*Delta)-s)*(b0+b1*w+b2*w*w);
    }
    
    double r = 0.0 ;
    if (sqrt(s)<=1.42)
    {
        r = atan(1.0/cotd)/PI*180.0;
    }
    
    
	return r;
}





double	PipiAmp::eta22(double s) const
{
    
    double r = 1.0 ;
    if ((sqrt(s)>1.05)&&(sqrt(s)<=1.42))
    {
        double e = 0.2;
        
        r = 1.0-e*pow(1.0-1.05*1.05/s,3.0) ;
    }
    
	return r;
}






complex<double>	PipiAmp::t22(double s) const
{
	complex<double> r (0.0,0.0);
    
    if (s<=4.0*mpi*mpi)
    {
        double b0 =  2.4*1000.0;
        double b1 =  7.8*1000.0;
        double b2 =  23.7*1000.0;
        double Delta = 0.196;
        
        complex<double> kpi (0.0,0.0);
        complex<double> w (0.0,0.0);
        complex<double> cotd (0.0,0.0);
        
        if (s>=0.0)
        {
            kpi = aIm*sqrt(4.0*mpi*mpi-s)/2.0;
            w = (sqrt(s) - sqrt(1.05*1.05-s))/(sqrt(s) + sqrt(1.05*1.05-s));
            cotd = 1.0/aIm/sqrt(4.0*mpi*mpi/s-1.0)/pow(kpi,4)*pow(mpi,4)*s/(4.0*(mpi*mpi+Delta*Delta)-s)*(b0+b1*w+b2*w*w);
            r = 1.0/aIm/sqrt(4.0*mpi*mpi/s-1.0)/(cotd - aIm);
        }
        else
        {
            kpi = aIm*sqrt(4.0*mpi*mpi-s)/2.0;
            w = (aIm*sqrt(-s) - sqrt(1.05*1.05-s))/(aIm*sqrt(-s) + sqrt(1.05*1.05-s));
            cotd = 1.0/sqrt(1.0-4.0*mpi*mpi/s)/pow(kpi,4)*pow(mpi,4)*s/(4.0*(mpi*mpi+Delta*Delta)-s)*(b0+b1*w+b2*w*w);
            r = 1.0/sqrt(1.0-4.0*mpi*mpi/s)/(cotd - aIm);
        }
    }
    else if (sqrt(s)<=1.42)
    {
        r = 1.0/sqrt(1.0-4.0*mpi*mpi/s)*(eta22(s)*exp(2.0*aIm*delta22(s)/180.0*PI)-1.0)/2.0/aIm;
    }
    
    
	return r;
}



// ============================== //








complex<double>	PipiAmp::thatIL(double s, int Is, int L) const
{
	complex<double> r (0.1e-15,0.1e-15);
    
    if ((Is==0)&&(L==0))
    {
        r = t00(s)/(s-mpi*mpi/2.0)*(s-sAD_0_0); // adler zero for eta decay //
    }
    else if ((Is==1)&&(L==1))
    {
          r = t11(s)/((s-4.0*mpi*mpi)/4.0);
    }
    else if ((Is==0)&&(L==2))
    {
          r = t20(s)/pow((s-4.0*mpi*mpi)/4.0,2);
    }
    else if ((Is==2)&&(L==0))
    {
        r = t02(s)/(s-2.0*mpi*mpi)*(s-sAD_2_0); // adler zero for eta decay //
    }
    else if ((Is==2)&&(L==2))
    {
        r = t22(s)/pow((s-4.0*mpi*mpi)/4.0,2);
    }
    
    
	return r;
}






// =================================== //






// -------------------------------------- //
// Peng Guo, Indiana University, Jan.2015 //
// -------------------------------------- //




